﻿// Program.cs
//
// Copyright(C) 2009 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// THIS SOURCE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//
// IN NO EVENT SHALL UBISYS TECHNOLOGIES BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOURCE CODE OR THE USE OR
// OTHER DEALINGS IN THE SOURCE CODE.
//
// Shows how to use the ubisys(R) 13.56 MHz RFID USB device in applications
// built with C# and the .NET framework. Easily adaptable to other languages
// like VB.NET, Power Shell, etc.
//
// Notice: This demo does not make use of threading to keep it simple. In a
// real application, consider using multiple threads: One for the user 
// interface, one for communication with the USB device.
//
// For a native C++ version, please contact support@ubisys.de

using System;
using System.Collections.Generic;
using System.Linq;
using ubisys;

namespace ConsoleDemoCS
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                System.Console.WriteLine("ubisys(R) 13.56 MHz RFID C# Demo Application");
                System.Console.WriteLine("Copyright(C) 2009 ubisys technologies GmbH, Duesseldorf, Germany.");
                System.Console.WriteLine("All rights reserved.");
                System.Console.WriteLine();

                List<string> ports;

                ubisys.Enumerator.Enumerate(out ports);
                ubisys.Enumerator.SelectUSBDevices(ref ports, 0x19a6, 0x0003);

                if (ports.Count == 0)
                {
                    System.Console.WriteLine("There is no suitable USB device connected.");
                    return;
                }

                System.Console.WriteLine("{0} devices currently attached:", ports.Count);
                System.Console.WriteLine();

                for (int i = 0; i < ports.Count; i++)
                {
                    UInt16 vendorID, productID;
                    string serialNumber;

                    if (!ubisys.Enumerator.GetUSBDeviceInformation(ports[i],
                        out vendorID, out productID, out serialNumber))
                        throw new ApplicationException();

                    System.Console.Write((i + 1) + ") VID = " + vendorID);
                    System.Console.Write(", PID = " + productID);
                    System.Console.Write(", S/N = " + serialNumber);
                    System.Console.WriteLine(", " + 
                        ubisys.Enumerator.GetPortNameFromDevicePath(ports[i]));
                }

                System.Console.WriteLine();

                int deviceNumber = 0;

                while ((deviceNumber == 0) || (deviceNumber > ports.Count()))
                {
                    try
                    {
                        System.Console.Write("Connect to which device (0 to exit)? ");
                        deviceNumber = System.Convert.ToInt32(System.Console.ReadLine().Trim(), 10);

                        if (deviceNumber == 0)
                            return;
                    }
                    catch (Exception)
                    {
                        System.Console.WriteLine("Invalid choice. Please try again.");
                    }
                }

                // Zero-based index
                deviceNumber--;

                System.Console.WriteLine("Connecting to {0}...",
                    ubisys.Enumerator.GetPortNameFromDevicePath(ports[deviceNumber]));
                                  
                ubisys.CommandExchange cx = new ubisys.CommandExchange();

                cx.Open(ubisys.Enumerator.GetPortNameFromDevicePath(ports[deviceNumber]));
                
                string[] response;

                if (cx.ExchangeRequest("ATI", out response))
                {
                    System.Console.WriteLine(response[0]);
                    System.Console.WriteLine(response[1]);
                }

                if (cx.ExchangeRequest("AT+S", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                if (cx.ExchangeRequest("AT+RF=0", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                if (cx.ExchangeRequest("AT+RF?", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                if (cx.ExchangeRequest("AT+RF=1", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                if (cx.ExchangeRequest("AT+SCAN1", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                System.Console.WriteLine("Scan mode active. Press any key to terminate.");

                while (!System.Console.KeyAvailable)
                {
                    if (cx.IsDataAvailable() && cx.GetResponse(out response))
                    {
                        foreach (string i in response)
                            System.Console.WriteLine(i);
                    }
                }

                if (cx.ExchangeRequest("AT+SCAN0", out response))
                {
                    foreach (string i in response)
                        System.Console.WriteLine(i);
                }

                System.Console.ReadKey();

                while (!System.Console.KeyAvailable)
                {
                    if (cx.ExchangeRequest("AT+I", out response))
                    {
                        foreach (string i in response)
                            System.Console.WriteLine(i);
                    }

                    if (cx.ExchangeRequest("AT+S", out response))
                    {
                        foreach (string i in response)
                            System.Console.WriteLine(i);
                    }
                }
            }
            catch (Exception e)
            {
                System.Console.Error.WriteLine("An exception occured:");
                System.Console.Error.WriteLine(e);
                throw;
            }
        }
    }
}
