﻿// WindowsDeviceManagement.cs
//
// Copyright(C) 2009 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// THIS SOURCE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//
// IN NO EVENT SHALL UBISYS TECHNOLOGIES BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOURCE CODE OR THE USE OR
// OTHER DEALINGS IN THE SOURCE CODE.
//
// Part of the ubisys(R) 13.56 MHz RFID USB stick demo application
//
// Provides WindowsDeviceManagement, an interface class that allows 
// invokation of the operating system's Device Management API. This is useful
// for enumeration of USB devices. The Enumerator class makes use of these 
// services to create a list of device paths corresponding to suitable 
// devices

using System;
using System.Runtime.InteropServices;

namespace ubisys
{

    // The WindowsDeviceManagement class provides an interface to the 
    // SetupDiXyz() functions of Windows, i.e. the Device Management API. 
    // This API allows enumerating devices, including USB devices. We 
    // will use this set of API functions to search the system for the 
    // ubisys 13.56MHz RFID Stick based on Vendor ID (VID) and 
    // Product ID (PID) information, which is unambiguous.
    sealed class WindowsDeviceManagement
    {
        // Constants
        public const short DIGCF_PRESENT = 0x00000002;
        public const short DIGCF_DEVICEINTERFACE = 0x00000010;
        static public IntPtr INVALID_HANDLE_VALUE = new IntPtr(-1);
        public const int ERROR_NO_MORE_ITEMS = 259;
        public const int ERROR_INSUFFICIENT_BUFFER = 122;

        // SP_DEVICE_INTERFACE_DATA structure
        [StructLayout(LayoutKind.Sequential)]
        public struct SP_DEVICE_INTERFACE_DATA
        {
            public UInt32 cbSize;
            public Guid InterfaceClassGuid;
            public UInt32 Flags;
            public UIntPtr Reserved;
        }

        // SP_DEVICE_INTERFACE_DETAIL_DATA structure
        [StructLayout(LayoutKind.Sequential)]
        public struct SP_DEVICE_INTERFACE_DETAIL_DATA
        {
            public UInt32 cbSize;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0)]
            public string DevicePath;
        }

        // SP_DEVICE_INTERFACE_DETAIL_DATA structure
        [StructLayout(LayoutKind.Sequential)]
        public struct SP_DEVINFO_DATA
        {
            public UInt32 cbSize;
            public Guid ClassGuid;
            public UInt32 DevInst;
            public UIntPtr Reserved;
        }

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern bool SetupDiDestroyDeviceInfoList(IntPtr DeviceInfoSet);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern int SetupDiEnumDeviceInterfaces(IntPtr DeviceInfoSet,
            IntPtr DeviceInfoData, ref Guid InterfaceClassGuid,
            UInt32 MemberIndex, ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern int SetupDiEnumDeviceInterfaces(IntPtr DeviceInfoSet,
            ref SP_DEVINFO_DATA DeviceInfoData, ref Guid InterfaceClassGuid,
            UInt32 MemberIndex, ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern IntPtr SetupDiGetClassDevs(ref Guid ClassGuid,
            [MarshalAs(UnmanagedType.LPTStr)] string Enumerator,
            IntPtr hwndParent, UInt32 Flags);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern bool SetupDiGetDeviceInterfaceDetail(IntPtr DeviceInfoSet,
            ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData,
            ref SP_DEVICE_INTERFACE_DETAIL_DATA DeviceInterfaceDetailData,
            UInt32 DeviceInterfaceDetailDataSize,
            out UInt32 RequiredSize,
            ref SP_DEVINFO_DATA DeviceInfoData);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern bool SetupDiGetDeviceInterfaceDetail(IntPtr DeviceInfoSet,
            ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData,
            IntPtr DeviceInterfaceDetailData,
            UInt32 DeviceInterfaceDetailDataSize,
            out UInt32 RequiredSize,
            ref SP_DEVINFO_DATA DeviceInfoData);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static public extern bool SetupDiGetDeviceInterfaceDetail(IntPtr DeviceInfoSet,
            ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData,
            IntPtr DeviceInterfaceDetailData,
            UInt32 DeviceInterfaceDetailDataSize,
            out UInt32 RequiredSize,
            IntPtr DeviceInfoData);
    }
}
